program AnalysiereTauschUndOnLineverhalten;

uses
  m7config, m7utillo, m7dec, m7time, m7disk, m7grplo, m7utilg;

const
  ConfigFile: string = 'm7com.cfg';

  AnaTage : word = 7;
  StatStunden : word = 1;
  StatMin : word = 60;
  Version : string = 'V1.3';
type
  AbsTyp = (AMaus, AFido, AZer, AUse);
const
  AbsStr : array[AbsTyp] of string[10] =
    ('MausNet', 'Fido', 'Z-Netz', 'UseNet');
  WochenFlag: boolean = false;
  PMFlag: boolean = false;
  MsgList: boolean = false;
var
  NetzHour,LokalHour: array[0..23] of word;
  NetzHourDiff,LokalHourDiff: array[0..23] of longint;
  NetzDiff,LokalDiff: longint;
  NetzMsgs,LokalMsgs: word;
  WochenTag: boolean;
  UsrOk: array[1..2000] of boolean;

  WHourTDiff,WHourADiff,THourADiff: array[0..23] of longint;
  WHourAnz,THourAnz: array[0..23] of word;
  PMAnz: word;
  WDiffT, WDiffA, TDiffA: longint;

procedure ParamCheck;
var
  p: word;
  ps: string;

  procedure Help;
  begin
    writeln('Aufruf mit ANATOL {Parameter1 .. ParameterN}');
    writeln;
    writeln('Parameter:');
    writeln('  /P      Statistik ber eigene PMs');
    writeln('  /H:n    Stundenzusammenfassung fr n Stunden (1-4 oder 6)');
    writeln('  /D:n    Statistik fr die letzten n Tage (inkl. heute) [', AnaTage, ']');
    writeln('  /W     + nur Woche Mo-Fr, - nur Wochenende Sa-So');
    writeln('  /L      Auflistung der ausgewerteten lokalen Msgs');
    writeln;
    writeln('Beispiele:');
    writeln('  ANATOL /D:7 /W+     AM-Laufzeiten, Mo-Fr');
    writeln('  ANATOL /P /H:4      PM-Laufzeiten, jeweils 4 Stunden zusammengefat');
    halt;
  end;

  procedure error;
  begin
    writeln('Falscher Parameter.');
    writeln;
    Help;
  end;

  function CheckVal: integer;
  var
    wert, err: integer;
  begin
    if ps[3] <> ':' then Error;
    val(CopyR(ps, 4), wert, err);
    if err>0 then Error;
    CheckVal := wert;
  end;

  function CheckNurLetter: boolean;
  begin
    if length(ps) > 2 then Error;
    CheckNurLetter := true;
  end;

  function CheckPlusMinus: boolean;
  var
    p: word;
  begin
    if length(ps) <> 3 then Error;
    p := pos(ps[3], '+-');
    if p = 0 then Error;
    CheckPlusMinus := p = 1;
  end;

begin
  for p := 1 to ParamCount do begin
    ps := ParamStr(p);
    UpString(ps);
    if pos(ps[1], '-/') = 0 then Error;
    case ps[2] of
      'D': AnaTage := CheckVal;
      'W': begin
             WochenFlag := true;
             WochenTag := CheckPlusMinus;
           end;
      'P': if CheckNurLetter then PMFlag := true;
      'L': if CheckNurLetter then MsgList := true;
      'H': begin
             StatStunden := CheckVal;
             if not (StatStunden in [1..4,6]) then Error;
             StatMin := StatStunden * 60;
           end;
(*
      'M': AnaMsg := CheckVal;
      'A': if CheckNurLetter then Sort := SoAlpha;
      'K': if CheckNurLetter then Sort := SoKB;
      'L': if CheckNurLetter then Sort := SoLeser;
      'G': GrpList := CheckPlusMinus;
      'N': NurNetGrp := CheckPlusMinus;
*)
      '?': Help;
      else Error;
    end;
  end;
end;
(*
procedure GrpInit;
var
  num: integer;
begin
  ResetGrpFile;
  write('GrpInit ');
  num := 0;
  Fillchar(GrpArr, sizeof(GrpArr), 0);
  with GrpArr[0] do begin
    StrDup(Name, 'FFENTLICH');
    IstImNetz := false;
    ok := not NurNetGrp;
  end;
  repeat
    GetGruppe;
    inc(num);
    with Gruppe do begin
      if num mod 8 = 0 then
        write(num:5, #8#8#8#8#8);
      StrDup(GrpArr[num].Name, Name);
      GrpArr[num].IstImNetz := IstImNetz;
      GrpArr[num].ok := (Name <> '') and (CopyL(Name, 3) <> 'ex-')
        and (not NurNetGrp or IstImNetz);
    end;
  until EofGrpFile;
  writeln(num:5);
  GrpMax := num;
end;
*)
procedure PMstat;
type
  AngekommenArr = array[1..10000] of record
    num: word;
    date, time: integer;
  end;
  pAngekommenArr = ^AngekommenArr;
var
  pAng: pAngekommenArr;
  AngNum: word;
  pb: pParameterblock;

  procedure NetLogInit;
  (*
  03:44:10>S51429@W2       A 08.03.98 18:53 -> #4341
           1         2         3         4         5
  12345678901234567890123456789012345678901234567890
  *)
  type
    tTextBuf = array[0..16383] of byte;
    pTextBuf = ^tTextBuf;
  var
    fname, z: string;
    tag: integer;
    f: text;
    MausName, numS: string[8];
    num: word;
    date, time, err: integer;
    TextBuf: pTextBuf;
  begin
    write('NetLogInit ');
    new(TextBuf);
    AngNum := 0;
    num := 0;
    if LogPath = MausPath then LogPath := MausNetPath;
      { nicht umdefiniert, dann wird der Default stimmen }
    for tag := 0 to 7 do begin
      if tag > 0 then fname := 'MNLOG.' + WochenTagStr(tag)
      else fname := 'MAUSNET.LOG';
      assign(f, LogPath + fname);
      reset(f);
      SetTextBuf(f, TextBuf^);
      repeat
        readln(f, z);
        inc(num);
        if num mod 128 = 0 then write(num:5,#8#8#8#8#8);
        if (length(z) >= 50) and (z[10] = 'S') and (z[26] = 'A') then begin
          MausName := RTrim(Copy(z, 17, 8));
          if (MausName = MausVar.Kuerzel) and (z[46] = '#') then begin
            inc(AngNum);
            numS := Copy(z, 11, 5);
            with pAng^[AngNum] do begin
              val(nums, num, err);
              date := TageSeit(Copy(z, 28, 8));
              time := MinutenSeit(Copy(z, 37, 5));
              { writeln(Copy(z, 10, 32), ' ', num:5, ' ',
                      DatumSeit(date), ' ', ZeitSeit(time)); }
            end;
          end;
        end;
      until eof(f);
      close(f);
    end;
    writeln(num);
  end;

  procedure SucheLStat;
  var
    n: word;
  begin
    n := 1;
    while n <= AngNum do begin
      if pAng^[n].num = pb^.NetId.Nummer then begin
        pb^.LStatus := 'A';
        pb^.LDatum := pAng^[n].date;
        pb^.LZeit := pAng^[n].time;
        (* with pAng^[n] do
          writeln(num:5, ' ', DatumSeit(date), ' ', ZeitSeit(time)); *)
        break;
      end;
      inc(n);
    end;
  end;

var
  dz: pDaten;
  adr: tAdrHeader;
  VonAdr: string;
  p: word;
  Abs: AbsTyp;
  Num: word;
  DiffT, DiffN: integer;
  ok, VonHier: boolean;
begin { PMstat }
  new(pAng);
  NetLogInit;
  write('MsgInit ');
  ResetParameter('P');
  pb := GetParameterDirect('P', 'D', GetDateInt - AnaTage + 1);
  num := 0;
  if MsgList then begin
    assign(output, '');
    rewrite(output);
    writeln;
    writeln('Id      Von                  Geschrieben  Eingetauscht Diff  Angekommen   Diff');
    writeln;
  end;
  FillChar(WHourTDiff, sizeof(WHourTDiff), 0);
  FillChar(WHourADiff, sizeof(WHourADiff), 0);
  FillChar(THourADiff, sizeof(THourADiff), 0);
  FillChar(WHourAnz, sizeof(WHourAnz), 0);
  FillChar(THourAnz, sizeof(THourAnz), 0);
  PMAnz := 0;
  WDiffT := 0; WDiffA := 0; TDiffA := 0;
  while pb <> nil do begin
    if ParOk(pb) then begin
      inc(Num);
      if not MsgList then
        if Num mod 64 = 0 then
          write(Num:5, #8#8#8#8#8);
      if (pb^.NetId.Box = MausVar.BoxNum) and (pb^.Sendernummer > 0) then begin
        if pb^.LStatus <> 'A' then
          SucheLStat; { setzt LStatus um! }
        if pb^.LStatus = 'A' { von hier und angekommen } then begin
          dz := GetData(pb);
          GetAdrHeader(dz, adr);
          { writeln(pb^.Nummer:5, ' ', adr.Von, '->', adr.An); }
          { writeln(Num:5, ' ', Adr); }
          with adr do begin
            DiffT := (pb^.EDatum * 24 * 60 + pb^.EZeit)
                   - (WDatum * 24 * 60 + WZeit);
            DiffN := (pb^.LDatum * 24 * 60 + pb^.LZeit)
                   - (pb^.EDatum * 24 * 60 + pb^.EZeit);
            if DiffT < 0 then DiffT := 0; { min 0 }
            if DiffN < 0 then DiffN := 0; { min 0 }
            if MsgList then
              writeln('P', pb^.NetId.Nummer:5, '  ', AlignL(Von, 20), ' ', CopyL(DatumSeit(WDatum),5), ' ', ZeitSeit(WZeit),
                  '  ', CopyL(DatumSeit(pb^.EDatum),5), ' ', ZeitSeit(pb^.EZeit), DiffT/60:6:1,
                  '  ', CopyL(DatumSeit(pb^.LDatum),5), ' ', ZeitSeit(pb^.LZeit), DiffN/60:6:1);
            inc(WHourTDiff[WZeit div StatMin], DiffT);
            inc(WHourADiff[WZeit div StatMin], DiffN+DiffT);
            inc(WHourAnz[WZeit div StatMin]);
            inc(WDiffT, DiffT);
            inc(WDiffA, DiffT+DiffN);
            inc(THourADiff[pb^.EZeit div StatMin], DiffN);
            inc(THourAnz[pb^.EZeit div StatMin]);
            inc(TDiffA, DiffN);
            inc(PMAnz);
          end;
        end;
      end;
    end;
    pb := GetParameter;
  end;
  if not MsgList then
    writeln(Num:5)
  else
    writeln;
  dispose(pAng);
end;

procedure PMshow;
var
  h, h1, h2, n1, n2: word;
begin
  if not MsgList then begin
    assign(output, '');
    rewrite(output);
  end;
  write  ('ANATOL ', Version, ' fr die letzten ', AnaTage, ' Tage');
(*
  if WochenFlag then
    if WochenTag then write(' (Mo-Fr)')
    else write(' (Sa-So)'); *)
  writeln;
  writeln;
(*
  writeln('Mittlere Laufzeit: ', NetzDiff / NetzMsgs / 60:0:1, ' h',
          '  lokal: ', LokalDiff / LokalMsgs / 60:0:1, ' h');
*)
  if PMAnz = 0 then exit;
  writeln(PMAnz, ' ausgewertete PMs der MAUS ', MausVar.Kuerzel, ' ins MausNet');
  writeln;
  writeln('Mittl. Zeit vom Schreiben bis zum Tausch: ', WDiffT/PMAnz/60:0:1, ' h');
  writeln('                          bis zur Ankunft in der Ziel-MAUS: ', WDiffA/PMAnz/60:0:1, 'h');
  writeln('            vom Tausch bis zur Ankunft in der Ziel-MAUS: ', TDiffA/PMAnz/60:0:1,'h');
  writeln;
  writeln('Geschrieben   Anzahl   Tausch Ankunft   Geschrieben   Anzahl   Tausch Ankunft');
  writeln('-----------------------------------------------------------------------------');
  for h := 1 to 12 div StatStunden do begin
    h1 := (h-1) * StatStunden;
    n1 := h-1;
    h2 := h1 + 12;
    n2 := h2 div StatStunden;
    write(FormatStr(h1),':00-',FormatStr(h1+StatStunden-1), ':59', WHourAnz[n1]:4, WHourAnz[n1]/PMAnz*100:5:1, '%');
    if WHourAnz[n1] > 0 then
      write(WHourTDiff[n1]/WHourAnz[n1]/60:6:1, 'h', WHourADiff[n1]/WHourAnz[n1]/60:7:1, 'h')
    else
      write('':15);
    write(h2:6,':00-',h2+StatStunden-1, ':59', WHourAnz[n2]:4, WHourAnz[n2]/PMAnz*100:5:1, '%');
    if WHourAnz[n2] > 0 then
      write(WHourTDiff[n2]/WHourAnz[n2]/60:6:1, 'h', WHourADiff[n2]/WHourAnz[n2]/60:7:1, 'h');
    writeln;
  end;
  writeln;
  writeln('Getauscht     Anzahl   Ankunft          Getauscht     Anzahl   Ankunft');
  writeln('---------------------------------------------------------------------');
  for h := 1 to 12 div StatStunden do begin
    h1 := (h-1) * StatStunden;
    n1 := h-1;
    h2 := h1 + 12;
    n2 := h2 div StatStunden;
    write(FormatStr(h1),':00-',FormatStr(h1+StatStunden-1), ':59', THourAnz[n1]:4, THourAnz[n1]/PMAnz*100:5:1, '%');
    if THourAnz[n1] > 0 then
      write(THourADiff[n1]/THourAnz[n1]/60:6:1, 'h')
    else
      write('':7);
    write(h2:14,':00-',h2+StatStunden-1, ':59', THourAnz[n2]:4, THourAnz[n2]/PMAnz*100:5:1, '%');
    if THourAnz[n2] > 0 then
      write(THourADiff[n2]/THourAnz[n2]/60:6:1, 'h');
    writeln;
  end;
  writeln;
end;

procedure UsrInit;
var
  num: integer;
  bp: pBenutzerprofil;
  g: word;
begin
  ResetUserLong;
  write('UsrInit ');
  num := 0;
  FillChar(UsrOk, sizeof(UsrOk), false);
  repeat
    bp := GetUserLong;
    inc(num);
    if num mod 8 = 0 then
      write(num:5, #8#8#8#8#8);
    UsrOk[num] := (bp^.Geschlecht <> 'P');
  until bp=nil;
  writeln(num:5);
end;

(*
function GrpComp(a, b: integer): boolean;
begin
  case Sort of
    SoAlpha: GrpComp := GrpArr[GrpIdx[a]].Name^ <= GrpArr[GrpIdx[b]].Name^;
    SoLeser: GrpComp := GrpArr[GrpIdx[a]].Leser >= GrpArr[GrpIdx[b]].Leser;
    SoKBpro: GrpComp := GrpArr[GrpIdx[a]].KBpro >= GrpArr[GrpIdx[b]].KBpro;
    SoKB   : GrpComp := GrpArr[GrpIdx[a]].size  >= GrpArr[GrpIdx[b]].size;
  end;
end;

procedure GrpSwap(a, b: integer);
var
  x: word;
begin
  x := GrpIdx[a]; GrpIdx[a] := GrpIdx[b]; GrpIdx[b] := x;
end;
*)

procedure AMstat;
var
  pb: pParameterblock;
  dz: pDaten;
  adr: tAdrHeader;
  VonAdr: string;
  p: word;
  Abs: AbsTyp;
  Num: word;
  DiffMin: integer;
  ok, VonHier: boolean;
begin
  UsrInit;
  write('MsgInit ');
  ResetParameter('A');
  pb := GetParameterDirect('A', 'D', GetDateInt - AnaTage + 1);
  num := 0;
  LokalMsgs := 0; NetzMsgs := 0;
  LokalDiff := 0; NetzDiff := 0;
  FillChar(NetzHour, sizeof(NetzHour), 0);
  FillChar(LokalHour, sizeof(LokalHour), 0);
  FillChar(NetzHourDiff, sizeof(NetzHourDiff), 0);
  FillChar(LokalHourDiff, sizeof(LokalHourDiff), 0);
  if MsgList then begin
    assign(output, '');
    rewrite(output);
    writeln;
    writeln('Id      Von                  Geschrieben  Eingetauscht Diff');
    writeln;
  end;
  while pb <> nil do begin
    if ParOk(pb) then begin
      inc(Num);
      if (not MsgList) and (Num mod 64 = 0) then
        write(Num:5, #8#8#8#8#8);

      VonHier := (pb^.NetId.Box = MausVar.BoxNum);
      if VonHier then
        ok := (pb^.Sendernummer > 0) and UsrOk[pb^.Sendernummer]
      else
        ok := true;

      if ok then begin
        dz := GetData(pb);
        GetAdrHeader(dz, adr);
        if not VonHier then begin
          {--- MAUS-Msg? }
          p := pos('@', adr.Von);
          if p = 0 then ok := false;
          if ok then begin
            VonAdr := LTrim(CopyR(adr.Von, p+1));
            UpString(VonAdr);
          { writeln(Num:5, ' ', Adr); }
            if length(VonAdr) < 4 then
              Abs := AMaus
            else if pos('FIDO', VonAdr) > 0 then Abs := AFido
            else if pos('.ZER', VonAdr) > 0 then Abs := AZer
            else Abs := AUse;
            ok := (Abs = AMaus);
          end;
        end;

        if ok and WochenFlag then
          ok := (WochenTag = (Copy(WochenTagStr(Adr.WDatum), 1, 1) <> 'S'));

        if ok then begin
          with Adr do begin
            DiffMin := (pb^.EDatum * 24 * 60 + pb^.EZeit) - (WDatum * 24 * 60 + WZeit);
            if DiffMin < 0 then DiffMin := 0; { min 0 }
            if VonHier then begin
              if MsgList then
                writeln('A', pb^.NetId.Nummer:5, '  ', AlignL(Von, 20), ' ', CopyL(DatumSeit(WDatum),5), ' ', ZeitSeit(WZeit),
                    '  ', CopyL(DatumSeit(pb^.EDatum),5), ' ', ZeitSeit(pb^.EZeit), DiffMin/60:6:1);
              inc(LokalDiff, DiffMin);
              inc(LokalHour[WZeit div StatMin]);
              inc(LokalHourDiff[WZeit div StatMin], DiffMin);
              inc(LokalMsgs);
            end else begin
              inc(NetzDiff, DiffMin);
              inc(NetzHour[WZeit div StatMin]);
              inc(NetzHourDiff[WZeit div StatMin], DiffMin);
              inc(NetzMsgs);
            end;
          end;
        end;
      end;
    end;
    pb := GetParameter;
  end;
  writeln(Num:5);
end;

procedure AMshow;
var
  h, h1, h2, n1, n2: word;
begin
  if not MsgList then begin
    assign(output, '');
    rewrite(output);
  end;
  write  ('ANATOL ', Version, ' fr die letzten ', AnaTage, ' Tage');
  if WochenFlag then
    if WochenTag then write(' (Mo-Fr)')
    else write(' (Sa-So)');
  writeln;
  if (NetzMsgs = 0) or (LokalMsgs = 0) then exit;
  writeln;
  writeln('Allgemeine Msgs aus dem MausNet: ', NetzMsgs);
  writeln('Mittlere Laufzeit vom Schreiben bis zur Ankunft hier: ',
           NetzDiff / NetzMsgs / 60:0:1, ' h');
  writeln;
  writeln('Allgemeine Msgs aus der ', MausVar.Kuerzel,': ', LokalMsgs);
  writeln('Mittlere Verzgerung vom Schreiben bis zum Tausch: ',
           LokalDiff / LokalMsgs / 60:0:1, ' h');
  writeln;
  writeln('Geschrieben   Netz  Diff   Lokal Diff     Geschrieben   Netz  Diff   Lokal Diff');
  writeln('-------------------------------------------------------------------------------');
  for h := 1 to 12 div StatStunden do begin
    h1 := (h-1) * StatStunden;
    n1 := h-1;
    h2 := h1 + 12;
    n2 := h2 div StatStunden;
    write(FormatStr(h1),':00-',FormatStr(h1+StatStunden-1), ':59');
    write(NetzHour[n1]/NetzMsgs*100:6:1, '%', NetzHourDiff[n1]/NetzHour[n1]/60:5:1, 'h',
          LokalHour[n1]/LokalMsgs*100:6:1, '%');
    if LokalHour[n1] > 0 then write(LokalHourDiff[n1]/LokalHour[n1]/60:5:1, 'h')
      else write('':6);
    write('':5, FormatStr(h2),':00-',FormatStr(h2+StatStunden-1), ':59');
    write(NetzHour[n2]/NetzMsgs*100:6:1, '%', NetzHourDiff[n2]/NetzHour[n2]/60:5:1, 'h',
          LokalHour[n2]/LokalMsgs*100:6:1, '%');
    if LokalHour[n2] > 0 then write(LokalHourDiff[n2]/LokalHour[n2]/60:5:1, 'h');
    writeln;
  end;
  writeln;
end;

begin
  if ConfigFile = '' then halt;
  CloseConfig;
  writeln('ANATOL ', Version, ' - ANAlyseTauschandOnLinetime - Hilfe mit /?');
  writeln;
  ParamCheck;
  { GrpInit; }
  { UsrInit; }
  if PMflag then begin
    PMStat;
    PMShow;
  end else begin
    AMstat;
    AMshow;
  end;
end.
